]> git.xonotic.org Git - xonotic/darkplaces.git/blobdiff - cl_main.c
removed decals
[xonotic/darkplaces.git] / cl_main.c
index 73d7adc7ec3c6740d603a70f8ee8131a3ed1fc81..c2f433c61466dcfb59877ab16c2e8968a48765c9 100644 (file)
--- a/cl_main.c
+++ b/cl_main.c
@@ -49,6 +49,7 @@ cvar_t freelook = {CVAR_SAVE, "freelook", "1"};
 cvar_t cl_draweffects = {0, "cl_draweffects", "1"};
 
 mempool_t *cl_scores_mempool;
+mempool_t *cl_refdef_mempool;
 
 client_static_t        cls;
 client_state_t cl;
@@ -57,9 +58,6 @@ entity_t              cl_entities[MAX_EDICTS];
 entity_t               cl_static_entities[MAX_STATIC_ENTITIES];
 lightstyle_t   cl_lightstyle[MAX_LIGHTSTYLES];
 
-int                            cl_numvisedicts;
-entity_t               *cl_visedicts[MAX_VISEDICTS];
-
 typedef struct effect_s
 {
        int active;
@@ -108,8 +106,8 @@ void CL_ClearState (void)
        memset(cl_beams, 0, sizeof(cl_beams));
        memset(cl_dlights, 0, sizeof(cl_dlights));
        memset(cl_effect, 0, sizeof(cl_effect));
+       CL_Screen_NewMap();
        CL_Particles_Clear();
-       CL_Decals_Clear();
        // LordHavoc: have to set up the baseline info for alpha and other stuff
        for (i = 0;i < MAX_EDICTS;i++)
        {
@@ -117,29 +115,31 @@ void CL_ClearState (void)
                ClearStateToDefault(&cl_entities[i].state_previous);
                ClearStateToDefault(&cl_entities[i].state_current);
        }
+
+       CL_CGVM_Clear();
 }
 
-void CL_LerpUpdate(entity_t *e, int frame, int modelindex)
+void CL_LerpUpdate(entity_t *e)
 {
        entity_persistent_t *p;
        entity_render_t *r;
        p = &e->persistent;
        r = &e->render;
 
-       if (p->modelindex != modelindex)
+       if (p->modelindex != e->state_current.modelindex)
        {
                // reset all interpolation information
-               p->modelindex = modelindex;
-               p->frame1 = p->frame2 = frame;
+               p->modelindex = e->state_current.modelindex;
+               p->frame1 = p->frame2 = e->state_current.frame;
                p->frame1time = p->frame2time = cl.time;
                p->framelerp = 1;
        }
-       else if (p->frame2 != frame)
+       else if (p->frame2 != e->state_current.frame)
        {
                // transition to new frame
                p->frame1 = p->frame2;
                p->frame1time = p->frame2time;
-               p->frame2 = frame;
+               p->frame2 = e->state_current.frame;
                p->frame2time = cl.time;
                p->framelerp = 0;
        }
@@ -150,6 +150,9 @@ void CL_LerpUpdate(entity_t *e, int frame, int modelindex)
                p->framelerp = bound(0, p->framelerp, 1);
        }
 
+       r->model = cl.model_precache[e->state_current.modelindex];
+       Mod_CheckLoaded(r->model);
+       r->frame = e->state_current.frame;
        r->frame1 = p->frame1;
        r->frame2 = p->frame2;
        r->framelerp = p->framelerp;
@@ -167,6 +170,9 @@ This is also called on Host_Error, so it shouldn't cause any errors
 */
 void CL_Disconnect (void)
 {
+       if (cls.state == ca_dedicated)
+               return;
+
 // stop sounds (especially looping!)
        S_StopAllSounds (true);
 
@@ -262,11 +268,11 @@ static void CL_PrintEntities_f (void)
                        Con_Printf ("EMPTY\n");
                        continue;
                }
-               strncpy(name, ent->render.model->name, 30);
-               name[30] = 0;
-               for (j = strlen(name);j < 30;j++)
+               strncpy(name, ent->render.model->name, 25);
+               name[25] = 0;
+               for (j = strlen(name);j < 25;j++)
                        name[j] = ' ';
-               Con_Printf ("%s:%04i (%5i %5i %5i) [%3i %3i %3i]\n", name, ent->render.frame, (int) ent->render.origin[0], (int) ent->render.origin[1], (int) ent->render.origin[2], (int) ent->render.angles[0] % 360, (int) ent->render.angles[1] % 360, (int) ent->render.angles[2] % 360);
+               Con_Printf ("%s:%04i (%5i %5i %5i) [%3i %3i %3i] %4.2f %5.3f\n", name, ent->render.frame, (int) ent->render.origin[0], (int) ent->render.origin[1], (int) ent->render.origin[2], (int) ent->render.angles[0] % 360, (int) ent->render.angles[1] % 360, (int) ent->render.angles[2] % 360, ent->render.scale, ent->render.alpha);
        }
 }
 
@@ -324,10 +330,10 @@ static float CL_LerpPoint (void)
 static void CL_RelinkStaticEntities(void)
 {
        int i;
-       for (i = 0;i < cl.num_statics && cl_numvisedicts < MAX_VISEDICTS;i++)
+       for (i = 0;i < cl.num_statics && r_refdef.numentities < MAX_VISEDICTS;i++)
        {
                Mod_CheckLoaded(cl_static_entities[i].render.model);
-               cl_visedicts[cl_numvisedicts++] = &cl_static_entities[i];
+               r_refdef.entities[r_refdef.numentities++] = &cl_static_entities[i].render;
        }
 }
 
@@ -429,8 +435,8 @@ static void CL_RelinkNetworkEntities()
 
                        // if the delta is large, assume a teleport and don't lerp
                        VectorSubtract(ent->state_current.origin, ent->state_previous.origin, delta);
-                       // LordHavoc: increased tolerance from 100 to 200
-                       if ((sv.active && svs.maxclients == 1 && !(ent->state_current.flags & RENDER_STEP)) || cls.timedemo || DotProduct(delta, delta) > 200*200 || cl_nolerp.integer)
+                       // LordHavoc: increased tolerance from 100 to 200, and now to 1000
+                       if ((sv.active && svs.maxclients == 1 && !(ent->state_current.flags & RENDER_STEP)) || cls.timedemo || DotProduct(delta, delta) > 1000*1000 || cl_nolerp.integer)
                                lerp = 1;
                        else
                        {
@@ -460,15 +466,12 @@ static void CL_RelinkNetworkEntities()
 
                VectorCopy (neworg, ent->persistent.trail_origin);
                // persistent.modelindex will be updated by CL_LerpUpdate
-               if (ent->state_current.modelindex != ent->persistent.modelindex || !ent->state_previous.active)
+               if (ent->state_current.modelindex != ent->persistent.modelindex)
                        VectorCopy(neworg, oldorg);
 
                VectorCopy (neworg, ent->render.origin);
                ent->render.flags = ent->state_current.flags;
                ent->render.effects = effects = ent->state_current.effects;
-               ent->render.model = cl.model_precache[ent->state_current.modelindex];
-               Mod_CheckLoaded(ent->render.model);
-               ent->render.frame = ent->state_current.frame;
                if (cl.scores == NULL || !ent->state_current.colormap)
                        ent->render.colormap = -1; // no special coloring
                else
@@ -476,11 +479,9 @@ static void CL_RelinkNetworkEntities()
                ent->render.skinnum = ent->state_current.skin;
                ent->render.alpha = ent->state_current.alpha * (1.0f / 255.0f); // FIXME: interpolate?
                ent->render.scale = ent->state_current.scale * (1.0f / 16.0f); // FIXME: interpolate?
-               glowsize = ent->state_current.glowsize * 4.0f; // FIXME: interpolate?
-               glowcolor = ent->state_current.glowcolor;
 
                // update interpolation info
-               CL_LerpUpdate(ent, ent->state_current.frame, ent->state_current.modelindex);
+               CL_LerpUpdate(ent);
 
                // handle effects now...
                dlightradius = 0;
@@ -502,9 +503,9 @@ static void CL_RelinkNetworkEntities()
                                v2[0] = v[0] * 18 + neworg[0];
                                v2[1] = v[1] * 18 + neworg[1];
                                v2[2] = v[2] * 18 + neworg[2] + 16;
-                               TraceLine(neworg, v2, v, NULL, 0);
+                               TraceLine(neworg, v2, v, NULL, 0, true);
 
-                               CL_AllocDlight (NULL, v, 100, 1, 1, 1, 0, 0.1);
+                               CL_AllocDlight (NULL, v, 100, 1, 1, 1, 0, 0);
                        }
                        if (effects & EF_DIMLIGHT)
                        {
@@ -585,9 +586,10 @@ static void CL_RelinkNetworkEntities()
                                else if (ent->render.model->flags & EF_ROCKET)
                                {
                                        CL_RocketTrail (oldorg, ent->render.origin, 0, ent);
-                                       dlightcolor[0] += 200.0f;
-                                       dlightcolor[1] += 160.0f;
-                                       dlightcolor[2] +=  80.0f;
+                                       // LordHavoc: changed from 200, 160, 80 to 250, 200, 100
+                                       dlightcolor[0] += 250.0f;
+                                       dlightcolor[1] += 200.0f;
+                                       dlightcolor[2] += 100.0f;
                                }
                                else if (ent->render.model->flags & EF_GRENADE)
                                {
@@ -601,6 +603,8 @@ static void CL_RelinkNetworkEntities()
                        }
                }
                // LordHavoc: customizable glow
+               glowsize = ent->state_current.glowsize * 4.0f; // FIXME: interpolate?
+               glowcolor = ent->state_current.glowcolor;
                if (glowsize)
                {
                        byte *tempcolor = (byte *)&d_8to24table[glowcolor];
@@ -621,7 +625,7 @@ static void CL_RelinkNetworkEntities()
                        // hack to make glowing player light shine on their gun
                        if (i == cl.viewentity && !chase_active.integer)
                                vec[2] += 30;
-                       CL_AllocDlight (&ent->render, vec, dlightradius, dlightcolor[0] * d, dlightcolor[1] * d, dlightcolor[2] * d, 0, 0);
+                       CL_AllocDlight (/*&ent->render*/ NULL, vec, dlightradius, dlightcolor[0] * d, dlightcolor[1] * d, dlightcolor[2] * d, 0, 0);
                }
 
                if (chase_active.integer)
@@ -639,8 +643,8 @@ static void CL_RelinkNetworkEntities()
                        continue;
                if (effects & EF_NODRAW)
                        continue;
-               if (cl_numvisedicts < MAX_VISEDICTS)
-                       cl_visedicts[cl_numvisedicts++] = ent;
+               if (r_refdef.numentities < MAX_VISEDICTS)
+                       r_refdef.entities[r_refdef.numentities++] = &ent->render;
        }
 }
 
@@ -747,13 +751,13 @@ static void CL_RelinkEffects()
 
 void CL_RelinkEntities (void)
 {
-       cl_numvisedicts = 0;
-
+       CL_DecayLights ();
        CL_LerpPlayerVelocity();
        CL_RelinkNetworkEntities();
+       TraceLine_ScanForBModels();
        CL_RelinkEffects();
        CL_MoveParticles();
-       CL_UpdateDecals();
+       CL_UpdateTEnts();
 }
 
 
@@ -792,8 +796,14 @@ int CL_ReadFromServer (void)
        if (netshown)
                Con_Printf ("\n");
 
-       CL_RelinkEntities ();
-       CL_UpdateTEnts ();
+       r_refdef.numentities = 0;
+       if (cls.state == ca_connected && cl.worldmodel)
+       {
+               CL_RelinkEntities ();
+
+               // run cgame code (which can add more entities)
+               CL_CGVM_Frame();
+       }
 
 //
 // bring the links up to date
@@ -915,6 +925,12 @@ CL_Init
 */
 void CL_Init (void)
 {
+       cl_scores_mempool = Mem_AllocPool("client player info");
+
+       cl_refdef_mempool = Mem_AllocPool("refdef");
+       memset(&r_refdef, 0, sizeof(r_refdef));
+       r_refdef.entities = Mem_Alloc(cl_refdef_mempool, sizeof(entity_render_t *) * MAX_VISEDICTS);
+
        SZ_Alloc (&cls.message, 1024, "cls.message");
 
        CL_InitInput ();
@@ -965,11 +981,10 @@ void CL_Init (void)
        if (gamemode == GAME_NEHAHRA)
                Cmd_AddCommand ("pmodel", CL_PModel_f);
 
-       cl_scores_mempool = Mem_AllocPool("client player info");
-
        Cvar_RegisterVariable(&cl_draweffects);
 
        CL_Parse_Init();
        CL_Particles_Init();
-       CL_Decals_Init();
+       CL_Screen_Init();
+       CL_CGVM_Init();
 }